@@ -13,18 +13,14 @@ module Sequel
       # ORA-03114: not connected to ORACLE
       CONNECTION_ERROR_CODES = [ 28, 1012, 3113, 3114 ]      
       
-      ORACLE_TYPES = {:blob=>lambda{|b| Sequel::SQL::Blob.new(b.read)}}
+      ORACLE_TYPES = {
+        :blob=>lambda{|b| Sequel::SQL::Blob.new(b.read)},
+        :clob=>lambda{|b| b.read}
+      }
 
       # Hash of conversion procs for this database.
       attr_reader :conversion_procs
 
-      def initialize(opts={})
-        super
-        @autosequence = opts[:autosequence]
-        @primary_key_sequences = {}
-        @conversion_procs = ORACLE_TYPES.dup
-      end
-
       def connect(server)
         opts = server_opts(server)
         if opts[:database]
@@ -34,7 +30,9 @@ module Sequel
           dbname = opts[:host]
         end
         conn = OCI8.new(opts[:user], opts[:password], dbname, opts[:privilege])
-        conn.prefetch_rows = typecast_value_integer(opts[:prefetch_rows]) if opts[:prefetch_rows]
+        if prefetch_rows = opts.fetch(:prefetch_rows, 100)
+          conn.prefetch_rows = typecast_value_integer(prefetch_rows)
+        end
         conn.autocommit = true
         conn.non_blocking = true
         
@@ -57,18 +55,23 @@ module Sequel
         conn
       end
 
-      def execute(sql, opts={}, &block)
+      def disconnect_connection(c)
+        c.logoff
+      rescue OCIInvalidHandle
+        nil
+      end
+
+      def execute(sql, opts=OPTS, &block)
         _execute(nil, sql, opts, &block)
       end
-      alias do execute
 
-      def execute_insert(sql, opts={})
+      def execute_insert(sql, opts=OPTS)
         _execute(:insert, sql, opts)
       end
 
       private
 
-      def _execute(type, sql, opts={}, &block)
+      def _execute(type, sql, opts=OPTS, &block)
         synchronize(opts[:server]) do |conn|
           begin
             return execute_prepared_statement(conn, type, sql, opts, &block) if sql.is_a?(Symbol)
@@ -98,6 +101,12 @@ module Sequel
         end
       end
 
+      def adapter_initialize
+        @autosequence = @opts[:autosequence]
+        @primary_key_sequences = {}
+        @conversion_procs = ORACLE_TYPES.dup
+      end
+
       PS_TYPES = {'string'.freeze=>String, 'integer'.freeze=>Integer, 'float'.freeze=>Float,
         'decimal'.freeze=>Float, 'date'.freeze=>Time, 'datetime'.freeze=>Time,
         'time'.freeze=>Time, 'boolean'.freeze=>String, 'blob'.freeze=>OCI8::BLOB}
@@ -125,8 +134,34 @@ module Sequel
         end
       end
 
+      def connection_execute_method
+        :exec
+      end
+
+      def database_error_classes
+        [OCIException, RuntimeError]
+      end
+
+      def database_specific_error_class(exception, opts)
+        return super unless exception.respond_to?(:code)
+        case exception.code
+        when 1400, 1407
+          NotNullConstraintViolation
+        when 1
+          UniqueConstraintViolation
+        when 2291, 2292
+          ForeignKeyConstraintViolation
+        when 2290
+          CheckConstraintViolation
+        when 8177
+          SerializationFailure
+        else
+          super
+        end
+      end
+
       def execute_prepared_statement(conn, type, name, opts)
-        ps = prepared_statements[name]
+        ps = prepared_statement(name)
         sql = ps.prepared_sql
         if cursora = conn.prepared_statements[name]
           cursor, cursor_sql = cursora
@@ -136,11 +171,17 @@ module Sequel
           end
         end
         unless cursor
-          cursor = log_yield("Preparing #{name}: #{sql}"){conn.parse(sql)}
+          cursor = log_yield("PREPARE #{name}: #{sql}"){conn.parse(sql)}
           conn.prepared_statements[name] = [cursor, sql]
         end
         args = cursor_bind_params(conn, cursor, opts[:arguments])
-        r = log_yield("Executing #{name}", args){cursor.exec}
+        log_sql = "EXECUTE #{name}"
+        if ps.log_sql
+          log_sql << " ("
+          log_sql << sql
+          log_sql << ")"
+        end
+        r = log_yield(log_sql, args){cursor.exec}
         if block_given?
           yield(cursor)
         elsif type == :insert
@@ -170,20 +211,15 @@ module Sequel
         end
       end
 
-      def begin_transaction(conn, opts={})
+      def begin_transaction(conn, opts=OPTS)
         log_yield(TRANSACTION_BEGIN){conn.autocommit = false}
+        set_transaction_isolation(conn, opts)
       end
       
-      def commit_transaction(conn, opts={})
+      def commit_transaction(conn, opts=OPTS)
         log_yield(TRANSACTION_COMMIT){conn.commit}
       end
 
-      def disconnect_connection(c)
-        c.logoff
-      rescue OCIInvalidHandle
-        nil
-      end
-
       def disconnect_error?(e, opts)
         super || (e.is_a?(::OCIError) && CONNECTION_ERROR_CODES.include?(e.code))
       end
@@ -212,11 +248,11 @@ module Sequel
         super
       end
       
-      def rollback_transaction(conn, opts={})
+      def rollback_transaction(conn, opts=OPTS)
         log_yield(TRANSACTION_ROLLBACK){conn.rollback}
       end
 
-      def schema_parse_table(table, opts={})
+      def schema_parse_table(table, opts=OPTS)
         schema, table = schema_and_table(table)
         schema ||= opts[:schema]
         schema_and_table = if ds = opts[:dataset]
@@ -237,7 +273,7 @@ module Sequel
 
         # Default values
         defaults = begin
-          metadata_dataset.from(:dba_tab_cols).
+          metadata_dataset.from(:user_tab_cols).
             where(:table_name=>im.call(table)).
             to_hash(:column_name, :data_default)
         rescue DatabaseError
@@ -256,7 +292,7 @@ module Sequel
               :primary_key => pks.include?(column.name),
               :default => defaults[column.name],
               :oci8_type => column.data_type,
-              :db_type => column.type_string.split(' ')[0],
+              :db_type => column.type_string,
               :type_string => column.type_string,
               :charset_form => column.charset_form,
               :char_used => column.char_used?,
@@ -320,17 +356,17 @@ module Sequel
         
         # Run execute_select on the database with the given SQL and the stored
         # bind arguments.
-        def execute(sql, opts={}, &block)
+        def execute(sql, opts=OPTS, &block)
           super(prepared_sql, {:arguments=>bind_arguments}.merge(opts), &block)
         end
         
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_dui(sql, opts={}, &block)
+        def execute_dui(sql, opts=OPTS, &block)
           super(prepared_sql, {:arguments=>bind_arguments}.merge(opts), &block)
         end
         
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_insert(sql, opts={}, &block)
+        def execute_insert(sql, opts=OPTS, &block)
           super(prepared_sql, {:arguments=>bind_arguments}.merge(opts), &block)
         end
       end
@@ -342,17 +378,17 @@ module Sequel
           
         # Execute the stored prepared statement name and the stored bind
         # arguments instead of the SQL given.
-        def execute(sql, opts={}, &block)
+        def execute(sql, opts=OPTS, &block)
           super(prepared_statement_name, opts, &block)
         end
          
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_dui(sql, opts={}, &block)
+        def execute_dui(sql, opts=OPTS, &block)
           super(prepared_statement_name, opts, &block)
         end
           
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_insert(sql, opts={}, &block)
+        def execute_insert(sql, opts=OPTS, &block)
           super(prepared_statement_name, opts, &block)
         end
       end
@@ -364,6 +400,22 @@ module Sequel
         ps.call(bind_vars, &block)
       end
       
+      def fetch_rows(sql)
+        execute(sql) do |cursor|
+          cps = db.conversion_procs
+          cols = columns = cursor.get_col_names.map{|c| output_identifier(c)}
+          metadata = cursor.column_metadata
+          cm = cols.zip(metadata).map{|c, m| [c, cps[m.data_type]]}
+          @columns = columns
+          while r = cursor.fetch
+            row = {}
+            r.zip(cm).each{|v, (c, cp)| row[c] = ((v && cp) ? cp.call(v) : v)}
+            yield row
+          end
+        end
+        self
+      end
+
       # Prepare the given type of query with the given name and store
       # it in the database.  Note that a new native prepared statement is
       # created on each call to this prepared statement.
@@ -372,39 +424,7 @@ module Sequel
         ps.extend(PreparedStatementMethods)
         if name
           ps.prepared_statement_name = name
-          db.prepared_statements[name] = ps
-        end
-        ps
-      end
-      
-      def fetch_rows(sql)
-        execute(sql) do |cursor|
-          offset = @opts[:offset]
-          rn = row_number_column
-          cps = db.conversion_procs
-          cols = columns = cursor.get_col_names.map{|c| output_identifier(c)}
-          metadata = cursor.column_metadata
-          cm = cols.zip(metadata).map{|c, m| [c, cps[m.data_type]]}
-          columns = cols.reject{|x| x == rn} if offset
-          @columns = columns
-          while r = cursor.fetch
-            row = {}
-            r.zip(cm).each{|v, (c, cp)| row[c] = ((v && cp) ? cp.call(v) : v)}
-            row.delete(rn) if offset
-            yield row
-          end
-        end
-        self
-      end
-
-      # Create a named prepared statement that is stored in the
-      # database (and connection) for reuse.
-      def prepare(type, name=nil, *values)
-        ps = to_prepared_statement(type, values)
-        ps.extend(PreparedStatementMethods)
-        if name
-          ps.prepared_statement_name = name
-          db.prepared_statements[name] = ps
+          db.set_prepared_statement(name, ps)
         end
         ps
       end
