@@ -50,8 +50,8 @@ module Sequel
 
       # Commit an existing prepared transaction with the given transaction
       # identifier string.
-      def commit_prepared_transaction(transaction_id)
-        run("XA COMMIT #{literal(transaction_id)}")
+      def commit_prepared_transaction(transaction_id, opts=OPTS)
+        run("XA COMMIT #{literal(transaction_id)}", opts)
       end
 
       # MySQL uses the :mysql database type
@@ -112,8 +112,8 @@ module Sequel
 
       # Rollback an existing prepared transaction with the given transaction
       # identifier string.
-      def rollback_prepared_transaction(transaction_id)
-        run("XA ROLLBACK #{literal(transaction_id)}")
+      def rollback_prepared_transaction(transaction_id, opts=OPTS)
+        run("XA ROLLBACK #{literal(transaction_id)}", opts)
       end
 
       # Get version of MySQL server, used for determined capabilities.
@@ -129,12 +129,12 @@ module Sequel
         true
       end
       
-      # MySQL supports prepared transactions (two-phase commit) using XA
+      # MySQL 5+ supports prepared transactions (two-phase commit) using XA
       def supports_prepared_transactions?
         server_version >= 50000
       end
 
-      # MySQL supports savepoints
+      # MySQL 5+ supports savepoints
       def supports_savepoints?
         server_version >= 50000
       end
@@ -145,6 +145,14 @@ module Sequel
         super && (server_version <= 50512 || server_version >= 50523)
       end
 
+      # Support fractional timestamps on MySQL 5.6.5+ if the :fractional_seconds
+      # Database option is used.  Technically, MySQL 5.6.4+ supports them, but
+      # automatic initialization of datetime values wasn't supported to 5.6.5+,
+      # and this is related to that.
+      def supports_timestamp_usecs?
+        @supports_timestamp_usecs ||= server_version >= 50605 && typecast_value_boolean(opts[:fractional_seconds])
+      end
+
       # MySQL supports transaction isolation levels
       def supports_transaction_isolation_levels?
         true
@@ -185,7 +193,11 @@ module Sequel
             sql = super
             op[:table] = related
             op[:key] ||= primary_key_from_schema(related)
-            sql << ", ADD FOREIGN KEY (#{quote_identifier(op[:name])})#{column_references_sql(op)}"
+            sql << ", ADD "
+            if constraint_name = op.delete(:foreign_key_constraint_name)
+              sql << "CONSTRAINT #{quote_identifier(constraint_name)} "
+            end
+            sql << "FOREIGN KEY (#{quote_identifier(op[:name])})#{column_references_sql(op)}"
           else
             super
           end
@@ -292,9 +304,8 @@ module Sequel
       # Use XA START to start a new prepared transaction if the :prepare
       # option is given.
       def begin_transaction(conn, opts=OPTS)
-        if (s = opts[:prepare]) && (th = _trans(conn))[:savepoint_level] == 0
+        if (s = opts[:prepare]) && savepoint_level(conn) == 1
           log_connection_execute(conn, "XA START #{literal(s)}")
-          th[:savepoint_level] += 1
         else
           super
         end
@@ -315,7 +326,7 @@ module Sequel
       # Prepare the XA transaction for a two-phase commit if the
       # :prepare option is given.
       def commit_transaction(conn, opts=OPTS)
-        if (s = opts[:prepare]) && _trans(conn)[:savepoint_level] <= 1
+        if (s = opts[:prepare]) && savepoint_level(conn) <= 1
           log_connection_execute(conn, "XA END #{literal(s)}")
           log_connection_execute(conn, "XA PREPARE #{literal(s)}")
         else
@@ -401,6 +412,7 @@ module Sequel
       # Handle MySQL specific index SQL syntax
       def index_definition_sql(table_name, index)
         index_name = quote_identifier(index[:name] || default_index_name(table_name, index[:columns]))
+        raise Error, "Partial indexes are not supported for this database" if index[:where] && !supports_partial_indexes?
         index_type = case index[:type]
         when :full_text
           "FULLTEXT "
@@ -420,7 +432,7 @@ module Sequel
 
       # Rollback the currently open XA transaction
       def rollback_transaction(conn, opts=OPTS)
-        if (s = opts[:prepare]) && _trans(conn)[:savepoint_level] <= 1
+        if (s = opts[:prepare]) && savepoint_level(conn) <= 1
           log_connection_execute(conn, "XA END #{literal(s)}")
           log_connection_execute(conn, "XA PREPARE #{literal(s)}")
           log_connection_execute(conn, "XA ROLLBACK #{literal(s)}")
@@ -461,6 +473,12 @@ module Sequel
         end
       end
 
+      # Split DROP INDEX ops on MySQL 5.6+, as dropping them in the same
+      # statement as dropping a related foreign key causes an error.
+      def split_alter_table_op?(op)
+        server_version >= 50600 && (op[:op] == :drop_index || (op[:op] == :drop_constraint && op[:type] == :unique))
+      end
+
       # MySQL can combine multiple alter table ops into a single query.
       def supports_combining_alter_table_ops?
         true
@@ -495,7 +513,9 @@ module Sequel
       # MySQL has both datetime and timestamp classes, most people are going
       # to want datetime
       def type_literal_generic_datetime(column)
-        if column[:default] == Sequel::CURRENT_TIMESTAMP
+        if supports_timestamp_usecs?
+          :'datetime(6)'
+        elsif column[:default] == Sequel::CURRENT_TIMESTAMP
           :timestamp
         else
           :datetime
@@ -503,15 +523,28 @@ module Sequel
       end
 
       # MySQL has both datetime and timestamp classes, most people are going
-      # to want datetime
+      # to want datetime.
       def type_literal_generic_time(column)
-        column[:only_time] ? :time : type_literal_generic_datetime(column)
+        if column[:only_time]
+          if supports_timestamp_usecs?
+            :'time(6)'
+          else
+            :time
+          end
+        else
+          type_literal_generic_datetime(column)
+        end
       end
 
       # MySQL doesn't have a true boolean class, so it uses tinyint(1)
       def type_literal_generic_trueclass(column)
         :'tinyint(1)'
       end
+
+      # MySQL 5.0.2+ supports views with check option.
+      def view_with_check_option_support
+        :local if server_version >= 50002
+      end
     end
   
     # Dataset methods shared by datasets that use MySQL databases.
@@ -521,10 +554,6 @@ module Sequel
       COMMA_SEPARATOR = ', '.freeze
       FOR_SHARE = ' LOCK IN SHARE MODE'.freeze
       SQL_CALC_FOUND_ROWS = ' SQL_CALC_FOUND_ROWS'.freeze
-      DELETE_CLAUSE_METHODS = Dataset.clause_methods(:delete, %w'delete from where order limit')
-      INSERT_CLAUSE_METHODS = Dataset.clause_methods(:insert, %w'insert ignore into columns values on_duplicate_key_update')
-      SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'select distinct calc_found_rows columns from join where group having compounds order limit lock')
-      UPDATE_CLAUSE_METHODS = Dataset.clause_methods(:update, %w'update ignore table set where order limit')
       APOS = Dataset::APOS
       APOS_RE = Dataset::APOS_RE
       DOUBLE_APOS = Dataset::DOUBLE_APOS
@@ -563,7 +592,17 @@ module Sequel
       BACKSLASH_RE = /\\/.freeze
       QUAD_BACKSLASH = "\\\\\\\\".freeze
       BLOB_START = "0x".freeze
+      EMPTY_BLOB = "''".freeze
       HSTAR = "H*".freeze
+      CURRENT_TIMESTAMP_56 = 'CURRENT_TIMESTAMP(6)'.freeze
+
+      # Comes directly from MySQL's documentation, used for queries with limits without offsets
+      ONLY_OFFSET = ",18446744073709551615".freeze
+
+      Dataset.def_sql_method(self, :delete, %w'delete from where order limit')
+      Dataset.def_sql_method(self, :insert, %w'insert ignore into columns values on_duplicate_key_update')
+      Dataset.def_sql_method(self, :select, %w'select distinct calc_found_rows columns from join where group having compounds order limit lock')
+      Dataset.def_sql_method(self, :update, %w'update ignore table set where order limit')
 
       include Sequel::Dataset::Replace
 
@@ -608,6 +647,18 @@ module Sequel
         end
       end
       
+      # MySQL's CURRENT_TIMESTAMP does not use fractional seconds,
+      # even if the database itself supports fractional seconds. If
+      # MySQL 5.6.4+ is being used, use a value that will return
+      # fractional seconds.
+      def constant_sql_append(sql, constant)
+        if constant == :CURRENT_TIMESTAMP && supports_timestamp_usecs?
+          sql << CURRENT_TIMESTAMP_56
+        else
+          super
+        end
+      end
+
       # Use GROUP BY instead of DISTINCT ON if arguments are provided.
       def distinct(*args)
         args.empty? ? super : group(*args)
@@ -704,18 +755,16 @@ module Sequel
         clone(:on_duplicate_key_update => args)
       end
 
-      # MySQL specific syntax for inserting multiple values at once.
-      def multi_insert_sql(columns, values)
-        sql = LiteralString.new('VALUES ')
-        expression_list_append(sql, values.map{|r| Array(r)})
-        [insert_sql(columns, sql)]
-      end
-      
       # MySQL uses the nonstandard ` (backtick) for quoting identifiers.
       def quoted_identifier_append(sql, c)
         sql << BACKTICK << c.to_s.gsub(BACKTICK_RE, DOUBLE_BACKTICK) << BACKTICK
       end
 
+      # MySQL does not support derived column lists
+      def supports_derived_column_lists?
+        false
+      end
+
       # MySQL can emulate DISTINCT ON with its non-standard GROUP BY implementation,
       # though the rows returned cannot be made deterministic through ordering.
       def supports_distinct_on?
@@ -732,6 +781,11 @@ module Sequel
         false
       end
       
+      # MySQL does not support limits in correlated subqueries (or any subqueries that use IN).
+      def supports_limits_in_correlated_subqueries?
+        false
+      end
+    
       # MySQL supports modifying joined datasets
       def supports_modifying_joins?
         true
@@ -752,7 +806,7 @@ module Sequel
       # ignores them.  Also, using them seems to cause problems on 1.9.  Since
       # they are ignored anyway, not using them is probably best.
       def supports_timestamp_usecs?
-        false
+        db.supports_timestamp_usecs?
       end
       
       # Sets up the update methods to use UPDATE IGNORE.
@@ -767,11 +821,6 @@ module Sequel
       
       private
 
-      # MySQL supports the ORDER BY and LIMIT clauses for DELETE statements
-      def delete_clause_methods
-        DELETE_CLAUSE_METHODS
-      end
-      
       # Consider the first table in the joined dataset is the table to delete
       # from, but include the others for the purposes of selecting rows.
       def delete_from_sql(sql)
@@ -786,12 +835,6 @@ module Sequel
         end
       end
 
-      # MySQL supports the IGNORE and ON DUPLICATE KEY UPDATE clauses for INSERT statements
-      def insert_clause_methods
-        INSERT_CLAUSE_METHODS
-      end
-      alias replace_clause_methods insert_clause_methods
-
       # MySQL doesn't use the SQL standard DEFAULT VALUES.
       def insert_columns_sql(sql)
         values = opts[:values]
@@ -872,7 +915,11 @@ module Sequel
 
       # MySQL uses a preceding X for hex escaping strings
       def literal_blob_append(sql, v)
-        sql << BLOB_START << v.unpack(HSTAR).first
+        if v.empty?
+          sql << EMPTY_BLOB
+        else
+          sql << BLOB_START << v.unpack(HSTAR).first
+        end
       end
 
       # Use 0 for false on MySQL
@@ -899,11 +946,17 @@ module Sequel
         BOOL_TRUE
       end
       
-      # MySQL does not support the SQL WITH clause for SELECT statements
-      def select_clause_methods
-        SELECT_CLAUSE_METHODS
+      # MySQL supports multiple rows in INSERT.
+      def multi_insert_sql_strategy
+        :values
       end
-      
+
+      def select_only_offset_sql(sql)
+        sql << LIMIT
+        literal_append(sql, @opts[:offset])
+        sql << ONLY_OFFSET
+      end
+  
       # Support FOR SHARE locking when using the :share lock style.
       def select_lock_sql(sql)
         @opts[:lock] == :share ? (sql << FOR_SHARE) : super
@@ -914,11 +967,6 @@ module Sequel
         sql << SQL_CALC_FOUND_ROWS if opts[:calc_found_rows]
       end
 
-      # MySQL supports the ORDER BY and LIMIT clauses for UPDATE statements
-      def update_clause_methods
-        UPDATE_CLAUSE_METHODS
-      end
-
       # MySQL uses WITH ROLLUP syntax.
       def uses_with_rollup?
         true
