@@ -397,6 +397,24 @@ describe "DB#create_table" do
     @db.sqls.should == ["CREATE TABLE cats (name text, UNIQUE (name))"]
   end
 
+  specify "should accept partial index definitions" do
+    def @db.supports_partial_indexes?() true end
+    @db.create_table(:cats) do
+      integer :id
+      index :id, :where=>proc{id > 1}
+    end
+    @db.sqls.should == ["CREATE TABLE cats (id integer)", "CREATE INDEX cats_id_index ON cats (id) WHERE (id > 1)"]
+  end
+
+  specify "should raise an error if partial indexes are not supported" do
+    proc do 
+      @db.create_table(:cats) do
+        integer :id
+        index :id, :where=>proc{id > 1}
+      end
+    end.should raise_error(Sequel::Error)
+  end
+
   specify "should not raise on index error for unsupported index definitions if ignore_index_errors is used" do
     proc {
       @db.create_table(:cats, :ignore_index_errors=>true) do
@@ -683,6 +701,12 @@ describe "DB#create_table!" do
     @db.create_table!(:cats){|*a|}
     @db.sqls.should == ['DROP TABLE cats', 'CREATE TABLE cats ()']
   end
+  
+  specify "should use IF EXISTS if the database supports it" do
+    meta_def(@db, :supports_drop_table_if_exists?){true}
+    @db.create_table!(:cats){|*a|}
+    @db.sqls.should == ['DROP TABLE IF EXISTS cats', 'CREATE TABLE cats ()']
+  end
 end
 
 describe "DB#create_table?" do
@@ -757,6 +781,54 @@ describe "DB#create_join_table" do
   end
 end
   
+describe "DB#create_join_table?" do
+  before do
+    @db = Sequel.mock
+  end
+  
+  specify "should create the table if it does not already exist" do
+    meta_def(@db, :table_exists?){|a| false}
+    @db.create_join_table?(:cat_id=>:cats, :dog_id=>:dogs)
+    @db.sqls.should == ['CREATE TABLE cats_dogs (cat_id integer NOT NULL REFERENCES cats, dog_id integer NOT NULL REFERENCES dogs, PRIMARY KEY (cat_id, dog_id))', 'CREATE INDEX cats_dogs_dog_id_cat_id_index ON cats_dogs (dog_id, cat_id)']
+  end
+
+  specify "should not create the table if it already exists" do
+    meta_def(@db, :table_exists?){|a| true}
+    @db.create_join_table?(:cat_id=>:cats, :dog_id=>:dogs)
+    @db.sqls.should == []
+  end
+
+  specify "should use IF NOT EXISTS if the database supports it" do
+    meta_def(@db, :supports_create_table_if_not_exists?){true}
+    @db.create_join_table?(:cat_id=>:cats, :dog_id=>:dogs)
+    @db.sqls.should == ['CREATE TABLE IF NOT EXISTS cats_dogs (cat_id integer NOT NULL REFERENCES cats, dog_id integer NOT NULL REFERENCES dogs, PRIMARY KEY (cat_id, dog_id))', 'CREATE INDEX cats_dogs_dog_id_cat_id_index ON cats_dogs (dog_id, cat_id)']
+  end
+end
+  
+describe "DB#create_join_table!" do
+  before do
+    @db = Sequel.mock
+  end
+  
+  specify "should drop the table first if it already exists" do
+    meta_def(@db, :table_exists?){|a| true}
+    @db.create_join_table!(:cat_id=>:cats, :dog_id=>:dogs)
+    @db.sqls.should == ['DROP TABLE cats_dogs', 'CREATE TABLE cats_dogs (cat_id integer NOT NULL REFERENCES cats, dog_id integer NOT NULL REFERENCES dogs, PRIMARY KEY (cat_id, dog_id))', 'CREATE INDEX cats_dogs_dog_id_cat_id_index ON cats_dogs (dog_id, cat_id)']
+  end
+
+  specify "should not drop the table if it doesn't exists" do
+    meta_def(@db, :table_exists?){|a| false}
+    @db.create_join_table!(:cat_id=>:cats, :dog_id=>:dogs)
+    @db.sqls.should == ['CREATE TABLE cats_dogs (cat_id integer NOT NULL REFERENCES cats, dog_id integer NOT NULL REFERENCES dogs, PRIMARY KEY (cat_id, dog_id))', 'CREATE INDEX cats_dogs_dog_id_cat_id_index ON cats_dogs (dog_id, cat_id)']
+  end
+
+  specify "should use IF EXISTS if the database supports it" do
+    meta_def(@db, :supports_drop_table_if_exists?){true}
+    @db.create_join_table!(:cat_id=>:cats, :dog_id=>:dogs)
+    @db.sqls.should == ['DROP TABLE IF EXISTS cats_dogs', 'CREATE TABLE cats_dogs (cat_id integer NOT NULL REFERENCES cats, dog_id integer NOT NULL REFERENCES dogs, PRIMARY KEY (cat_id, dog_id))', 'CREATE INDEX cats_dogs_dog_id_cat_id_index ON cats_dogs (dog_id, cat_id)']
+  end
+end
+  
 describe "DB#drop_join_table" do
   before do
     @db = Sequel.mock
@@ -1327,6 +1399,13 @@ describe "Database#create_view" do
     @db.sqls.should == ['CREATE VIEW test (d, e) AS SELECT a, b FROM items ORDER BY c']
   end
 
+  specify "should handle :check option" do
+    @db.create_view :test, @db[:items].select(:a, :b).order(:c), :check=>true
+    @db.sqls.should == ['CREATE VIEW test AS SELECT a, b FROM items ORDER BY c WITH CHECK OPTION']
+    @db.create_view :test, @db[:items].select(:a, :b).order(:c), :check=>:local
+    @db.sqls.should == ['CREATE VIEW test AS SELECT a, b FROM items ORDER BY c WITH LOCAL CHECK OPTION']
+  end
+
   specify "should handle create_or_replace_view" do
     @db.create_or_replace_view :sch__test, "SELECT * FROM xyz"
     @db.sqls.should == ['DROP VIEW sch.test', 'CREATE VIEW sch.test AS SELECT * FROM xyz']
@@ -1361,10 +1440,15 @@ describe "Database#drop_view" do
     @db.sqls.should == ['DROP VIEW cats', 'DROP VIEW dogs']
   end
 
-  specify "should take an options hash and support the :cascade option" do
+  specify "should support the :cascade option" do
     @db.drop_view :cats, :dogs, :cascade=>true
     @db.sqls.should == ['DROP VIEW cats CASCADE', 'DROP VIEW dogs CASCADE']
   end
+
+  specify "should support the :if_exists option" do
+    @db.drop_view :cats, :dogs, :if_exists=>true
+    @db.sqls.should == ['DROP VIEW IF EXISTS cats', 'DROP VIEW IF EXISTS dogs']
+  end
 end
 
 describe "Database#alter_table_sql" do
@@ -1514,6 +1598,7 @@ describe "Schema Parser" do
     @db = Sequel.mock(:host=>'postgres')
     @db.extend(sm)
     @db.schema(:interval).first.last[:type].should == :interval
+    @db.schema(:citext).first.last[:type].should == :string
 
     @db = Sequel.mock(:host=>'mysql')
     @db.extend(sm)
