@@ -175,6 +175,11 @@ describe "Database index parsing" do
     DB.create_table!(:items){Integer :n; Integer :a; primary_key [:n, :a]}
     DB.indexes(:items).should == {}
   end
+
+  cspecify "should not include partial indexes", :sqlite do
+    DB.create_table!(:items){Integer :n; Integer :a; index :n, :where=>proc{n > 10}}
+    DB.indexes(:items).should == {}
+  end if DB.supports_partial_indexes?
 end if DB.supports_index_parsing?
 
 describe "Database foreign key parsing" do
@@ -201,7 +206,7 @@ describe "Database foreign key parsing" do
   end
 
   specify "should parse foreign key information into an array of hashes" do
-    @db.create_table!(:a, :engine=>:InnoDB){primary_key :c; Integer :d; index :d, :unique=>true}
+    @db.create_table!(:a, :engine=>:InnoDB){primary_key :c; Integer :d, :null => false, :unique => true}
     @db.create_table!(:b, :engine=>:InnoDB){foreign_key :e, :a}
     @pr[:a]
     @pr[:b, [[:e], :a, [:pk, :c]]]
@@ -212,7 +217,7 @@ describe "Database foreign key parsing" do
     @db.alter_table(:b){add_foreign_key [:f], :a, :key=>[:c]}
     @pr[:b, [[:e], :a, [:pk, :c]], [[:f], :a, [:c]], [[:f], :a, [:d]]]
 
-    @db.alter_table(:a){add_index [:d, :c], :unique=>true}
+    @db.alter_table(:a){add_unique_constraint [:d, :c]}
     @db.alter_table(:b){add_foreign_key [:f, :e], :a, :key=>[:d, :c]}
     @pr[:b, [[:e], :a, [:pk, :c]], [[:f], :a, [:c]], [[:f], :a, [:d]], [[:f, :e], :a, [:d, :c]]]
 
@@ -227,9 +232,9 @@ describe "Database foreign key parsing" do
   end
 
   specify "should handle composite foreign and primary keys" do
-    @db.create_table!(:a, :engine=>:InnoDB){Integer :b; Integer :c; primary_key [:b, :c]; index [:c, :b], :unique=>true}
-    @db.create_table!(:b, :engine=>:InnoDB){Integer :e; Integer :f; foreign_key [:e, :f], :a; foreign_key [:f, :e], :a, :key=>[:c, :b]}
-    @pr[:b, [[:e, :f], :a, [:pk, :b, :c]], [[:f, :e], :a, [:c, :b]]]
+    @db.create_table!(:a, :engine=>:InnoDB){Integer :b, :null=>false; Integer :c, :null=>false; Integer :d, :null=>false; primary_key [:b, :c]; unique [:d, :c]}
+    @db.create_table!(:b, :engine=>:InnoDB){Integer :e, :null=>false; Integer :f, :null=>false; Integer :g, :null=>false; foreign_key [:e, :f], :a; foreign_key [:g, :f], :a, :key=>[:d, :c]}
+    @pr[:b, [[:e, :f], :a, [:pk, :b, :c]], [[:g, :f], :a, [:d, :c]]]
   end
 end if DB.supports_foreign_key_parsing?
 
@@ -261,14 +266,20 @@ describe "Database schema modifiers" do
     @db[:items2].all.should == [{:number=>10}]
   end
   
+  specify "should not raise an error if table doesn't exist when using drop_table :if_exists" do
+    proc{@db.drop_table(:items, :if_exists=>true)}.should_not raise_error
+  end if DB.supports_drop_table_if_exists?
+
   describe "views" do
     before do
+      @db.drop_view(:items_view2) rescue nil
       @db.drop_view(:items_view) rescue nil
       @db.create_table(:items){Integer :number}
       @ds.insert(:number=>1)
       @ds.insert(:number=>2)
     end
     after do
+      @db.drop_view(:items_view2) rescue nil
       @db.drop_view(:items_view) rescue nil
     end
 
@@ -277,11 +288,46 @@ describe "Database schema modifiers" do
       @db[:items_view].map(:number).should == [1]
     end
 
+    specify "should create views with check options correctly" do
+      @db.create_view(:items_view, @ds.where{number > 2}, :check=>true)
+      proc{@db[:items_view].insert(1)}.should raise_error(Sequel::DatabaseError)
+      @db[:items_view].insert(3)
+      @db[:items_view].select_order_map(:number).should == [3]
+      @db.create_view(:items_view2, @db[:items_view].where{number > 1}, :check=>true)
+      proc{@db[:items_view2].insert(1)}.should raise_error(Sequel::DatabaseError)
+      proc{@db[:items_view2].insert(2)}.should raise_error(Sequel::DatabaseError)
+      @db[:items_view2].insert(4)
+      @db[:items_view2].select_order_map(:number).should == [3, 4]
+      @ds.select_order_map(:number).should == [1, 2, 3, 4]
+    end if DB.supports_views_with_check_option?
+
+    specify "should create views with local check options correctly" do
+      @db.create_view(:items_view, @ds.where{number > 2})
+      @db[:items_view].insert(3)
+      @db[:items_view].select_order_map(:number).should == [3]
+      @db.create_view(:items_view2, @db[:items_view].where{number > 1}, :check=>:local)
+      proc{@db[:items_view2].insert(1)}.should raise_error(Sequel::DatabaseError)
+      @db[:items_view2].insert(2)
+      @db[:items_view2].insert(4)
+      @db[:items_view2].select_order_map(:number).should == [3, 4]
+      @ds.select_order_map(:number).should == [1, 2, 2, 3, 4]
+    end if DB.supports_views_with_local_check_option?
+
     cspecify "should create views with explicit columns correctly", :sqlite do
       @db.create_view(:items_view, @ds.where(:number=>1), :columns=>[:n])
       @db[:items_view].map(:n).should == [1]
     end
 
+    specify "should drop views correctly" do
+      @db.create_view(:items_view, @ds.where(:number=>1))
+      @db.drop_view(:items_view)
+      proc{@db[:items_view].map(:number)}.should raise_error(Sequel::DatabaseError)
+    end
+
+    specify "should not raise an error if view doesn't exist when using drop_view :if_exists" do
+      proc{@db.drop_view(:items_view, :if_exists=>true)}.should_not raise_error
+    end if DB.supports_drop_table_if_exists?
+
     specify "should create or replace views correctly" do
       @db.create_or_replace_view(:items_view, @ds.where(:number=>1))
       @db[:items_view].map(:number).should == [1]
@@ -293,7 +339,7 @@ describe "Database schema modifiers" do
   specify "should handle create table in a rolled back transaction" do
     @db.drop_table?(:items)
     @db.transaction(:rollback=>:always){@db.create_table(:items){Integer :number}}
-    @db.table_exists?(:items).should be_false
+    @db.table_exists?(:items).should == false
   end if DB.supports_transactional_ddl?
   
   describe "join tables" do
@@ -352,6 +398,14 @@ describe "Database schema modifiers" do
     @ds.columns!.should == [:number]
   end
 
+  specify "should allow creating partial indexes with tables" do
+    @db.create_table!(:items){Integer :number; index :number, :where=>proc{number > 10}}
+    @db.table_exists?(:items).should == true
+    @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:number]
+    @ds.insert([10])
+    @ds.columns!.should == [:number]
+  end if DB.supports_partial_indexes?
+
   specify "should handle combination of default, unique, and not null" do
     @db.create_table!(:items){Integer :number, :default=>0, :null=>false, :unique=>true}
     @db.table_exists?(:items).should == true
@@ -387,7 +441,7 @@ describe "Database schema modifiers" do
     @ds.all.should == [{:number=>10, :name=>nil}]
   end
 
-  cspecify "should add primary key columns to tables correctly", :h2, :derby do
+  cspecify "should add primary key columns to tables correctly", :derby do
     @db.create_table!(:items){Integer :number}
     @ds.insert(:number=>10)
     @db.alter_table(:items){add_primary_key :id}
@@ -405,7 +459,7 @@ describe "Database schema modifiers" do
     proc{@ds.insert(10)}.should_not raise_error
   end
 
-  cspecify "should add foreign key columns to tables correctly", :hsqldb do
+  specify "should add foreign key columns to tables correctly" do
     @db.create_table!(:items){primary_key :id}
     @ds.insert
     i = @ds.get(:id)
@@ -532,7 +586,7 @@ describe "Database schema modifiers" do
   end
 
   specify "should add unnamed unique constraints and foreign key table constraints correctly" do
-    @db.create_table!(:items, :engine=>:InnoDB){Integer :id; Integer :item_id}
+    @db.create_table!(:items, :engine=>:InnoDB){Integer :id, :null => false; Integer :item_id, :null => false}
     @db.alter_table(:items) do
       add_unique_constraint [:item_id, :id]
       add_foreign_key [:id, :item_id], :items, :key=>[:item_id, :id]
@@ -594,18 +648,16 @@ describe "Database schema modifiers" do
     @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:id]
   end
 
-  cspecify "should remove foreign key columns from tables correctly", :h2, :mssql, :hsqldb do
-    # MySQL with InnoDB cannot drop foreign key columns unless you know the
-    # name of the constraint, see Bug #14347
-    @db.create_table!(:items, :engine=>:MyISAM) do
+  specify "should remove foreign key columns from tables correctly" do
+    @db.create_table!(:items, :engine=>:InnoDB) do
       primary_key :id
       Integer :i
       foreign_key :item_id, :items
     end
     @ds.insert(:i=>10)
-    @db.drop_column(:items, :item_id)
+    @db.alter_table(:items){drop_foreign_key :item_id}
     @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:id, :i]
-  end
+  end if DB.supports_foreign_key_parsing?
 
   specify "should remove multiple columns in a single alter_table block" do
     @db.create_table!(:items) do
