@@ -4,8 +4,8 @@ $: << File.join(File.dirname(__FILE__), '..','lib', 'sequel')
 require 'version'
 Dir.chdir(File.dirname(__FILE__))
 erb = ERB.new(File.read('layout.html.erb'))
-Dir['pages/*'].each do |page|
-  public_loc = "#{page.gsub(/\Apages\//, 'public/')}.html"
+Dir['pages/*.html.erb'].each do |page|
+  public_loc = "#{page.gsub(/\Apages\//, 'public/').gsub('.erb', '')}"
   content = ERB.new(File.read(page)).result(binding)
   title = File.basename(page)
   File.open(public_loc, 'wb'){|f| f.write(erb.result(binding))}
